/*
  ceil.h

  Header file to use in association with ceil.c

  R J Hogan, 10 May 1999
*/

#include <stdio.h>
#include <string.h>

#define CHIL_LAT  51.1445
#define CHIL_LON  -1.4370
#define CHIL_ALT  90.0

#define LC_WAVELENGTH 905

#define LC_MAX_GATES 752
#define LC_MAXLINE 127
#define LC_UNKNOWN 0
#define LC_METRES 1
#define LC_FEET 2
#define LC_GATE_SPACING 30

#define LC_OK 0
#define LC_ERROR -1
#define LC_NOTFOUND -2
#define LC_NOTDONE -3
#define LC_ENDOFBLOCK 1
#define LC_ENDOFDATA 2

#define FEET_TO_METRES 1

#define LCFeetToMetres(feet) ((feet) * 0.3048)

typedef struct {
  int year, month, day, hour, minute, second;
} LCTime;

typedef struct {
  int number, height1, height2, height3, units;
} LCCloudBases;

typedef struct {
  double alpha[LC_MAX_GATES]; /* extinction in 1/m, after Klett inversion */
  int ngates;
  short int range[LC_MAX_GATES]; /* meters */
  short int beta[LC_MAX_GATES]; /* counts*/
} LCBackscatterProfile;

typedef struct {
  char **argv;
  float starthour, endhour, minheight, maxheight, dhour;
  float stdn, noisefactor, duration;
  int argc;
  int averays, avegates, status, cloudgate;
  LCBackscatterProfile profile, _profile_hr;
  LCTime time;
  LCCloudBases cloud_bases;
  unsigned short int reqyear, reqmonth, reqday;
  char bydate, sethourrange, setheightrange;
  char verbose, dots;
} LCData;
  


int LCGetFullRay(FILE * ceilfile,LCTime * time,LCCloudBases * cloudbases,
		 LCBackscatterProfile * profile) ;
int LCGetCloudBases(FILE *ceilfile,LCTime *time,LCCloudBases *cloudbases);

int LCCleanRay(int * cloudgate,float * stdn,LCBackscatterProfile * profile);
int LCCustomCleanRay(int * cloudgate,float * stdn,
		     LCBackscatterProfile * profile,int n,float factor);
int LCGetAveragedRay(FILE *ceilfile,LCTime *time,LCCloudBases *cloudbases,
		     LCBackscatterProfile *aveprofile,int nrays);
int LCAverageRay(LCBackscatterProfile * profile_in,
		 LCBackscatterProfile * profile_out,int nave);
int LCKlettInversion(LCBackscatterProfile *profile,
		     float extinction_to_backscatter_ratio);
double LCOpticalDepth(LCBackscatterProfile * profile,
		      int iheight1,int iheight2);
int LCIndexFromHeight(LCBackscatterProfile *profile,short int range);
int LCGetDate(char *datestr, LCTime *time);

void LCUsage(LCData *data);
void LCOptions(LCData *data);
int LCInit(int argc, char **argv, LCData *data);
int LCAdvance(LCData *data);
int LCFetchRay(LCData *data);
void LCEnd(LCData *data);
