#include "chil_global.h"

#define RETURN(dest, src) if ((dest) == NULL) return(src); else return(strcpy((dest), (src)))
#define N_BITWISE_PARAMETERS 16

char *
CHILFieldName(const Ray *ray,int field,char * name) 
{
  char *field_names[] = { UNKNOWN_NAME , ZH_NAME , ZDR_NAME , 
			  LDR_NAME , PHIDP_NAME , VEL1_NAME , VEL2_NAME ,
			  WIDTH_H_NAME, WIDTH_V_NAME, UNKNOWN_NAME,
			  UNKNOWN_NAME, UNKNOWN_NAME, UNKNOWN_NAME,
			  UNKNOWN_NAME, UNKNOWN_NAME, VELSTD_NAME};
  /* Find if v and Zh are unque */
  uchar i,sum=0,n=0;
  switch (ray->field_type[field]) {
  case ZH:
    for (i=0;i < ray->nrawfields;i++) {
      if (ray->field_type[i] == ZH) {
	sum++;
	if (i==field) n=sum;
      }
    }
    if (sum < 2 || sum==n)
      RETURN(name,ZH_NAME);
    else
      RETURN(name,ZH2_NAME);
    break;
  case V1:case V2:
    for (i=0;i < ray->nrawfields;i++) {
      if (ray->field_type[i] == V1 || ray->field_type[i] == V2 ) {
	++sum;
	if (i==field) n=sum;
      }
    }
    if (sum < 2)
      RETURN(name,VEL_NAME);
    else if (sum==n)
      RETURN(name,VEL2_NAME);
    else
      RETURN(name,VEL1_NAME);
    break;
  default:
    if (ray->field_type[field] < N_BITWISE_PARAMETERS)
      RETURN(name,field_names[ray->field_type[field]]);
    else {
      if (name == NULL) {
	return UNKNOWN_NAME; 
      }
      else {
	sprintf(name,"var%d",ray->field_type[field]);
	return name;
      }   
    }
  }
}

char *
CHILFieldUnits(const Ray *ray,int field,char * units)
{
  char *field_units[] = { UNKNOWN_UNITS , ZH_UNITS , ZDR_UNITS , 
			  LDR_UNITS , PHIDP_UNITS , VEL1_UNITS , VEL2_UNITS ,
			  WIDTH_UNITS , WIDTH_UNITS, UNKNOWN_UNITS ,
			  UNKNOWN_UNITS , UNKNOWN_UNITS , UNKNOWN_UNITS ,
			  UNKNOWN_UNITS , UNKNOWN_UNITS , VELSTD_UNITS};
  if (ray->field_type[field] < N_BITWISE_PARAMETERS)
    RETURN(units,field_units[ray->field_type[field]]);
  else
    RETURN(units,field_units[0]);
}

char
*CHILFieldDescription(const Ray *ray,int field,char * desc)
{
  char *field_desc[] = { "Unknown variable" , "Radar reflectivity factor" , 
			 "Differential reflectivity" , "Linear depolarisation ratio",
			 "Differential phase shift",
			 "Doppler velocity (H)","Doppler velocity (H&V)",
			 "Doppler spectral width (H)",
			 "Doppler spectral width (V)", "", "", "", "", "", "", 
			 "Doppler velocity standard deviation" };
  uchar field_type = ray->field_type[field];
  uchar i, sum = 0;

  if (field_type < N_BITWISE_PARAMETERS) {
    /* If is only velocity, don't refer to polarisation */
    if (field_type == V1 || field_type == V2) {
      for (i = 0; i < ray->nrawfields; i++) {
	if (ray->field_type[i] == V1 || ray->field_type[i] == V2) {
	  ++sum;
	}
      }
      if (sum == 1) {
	RETURN(desc, "Doppler velocity");
      }
    }
    RETURN(desc, field_desc[ray->field_type[field]]);
  }
  else {
    RETURN(desc, field_desc[0]);
  }
}

char *
CHILSystemName(const RasterInfo *info,char *name)
{
  char *system_names[] = { "Unknown","CAMRa","Rabelais",
			   "Galileo","Singapore"};
  if (info->system_id < 5)
    RETURN(name,system_names[info->system_id]);
  else
    RETURN(name,system_names[0]);
}

char *
CHILScanType(const RasterInfo *info, char *type)
{
  char *scan_type[] = { "Unknown", "RHI", "PPI", "Slant-plane", "Fixed",
			"Calibration", "Manual" };
  if (info->scan_type <= TS_93)
    RETURN(type, scan_type[info->scan_type]);
  else
    RETURN(type, scan_type[0]);
}

char
*CHILExperimentCode(const RasterInfo *info, char *code)
{
  char *experiment_codes[] = { "??", "Sp", "Ol", "Gp", "??",
			       "Cr", "??", "??", "Ba", "Ec",
			       "Ml", "??", "Sp", "Hy", "Ss",
			       "It", "Tp", "Cf", "Xp", "Cl",
			       "Ca", "Fx", "AD", "Rm", "Bs",
			       "Cr", "Mf", "WT", "LA", "MC",
			       "Tb", "MD", "CT", "??", "??",
			       "??", "??", "??", "??", "Tc",
			       "Tr", "Ts", "N1", "Tm" };
  if (info->expt_id <= 44)
    RETURN(code, experiment_codes[info->expt_id]);
  else
    RETURN(code, experiment_codes[0]);
}


/* At some time in the future, these functions will detect the location of 
   places other than Chilbolton (Singapore etc) */
float
CHILLatitude(const RasterInfo *info)
{
  switch (info->system_id) {
  case UNKNOWN_SYSTEM:
    return 0;
  case SINGAPORE:
    return SINGAPORE_LAT;
  default:
    return CHIL_LAT;
  }
}

float
CHILLongitude(const RasterInfo *info)
{
  switch (info->system_id) {
  case UNKNOWN_SYSTEM:
    return 0;
  case SINGAPORE:
    return SINGAPORE_LON;
  default:
    return CHIL_LON;
  }
}

float
CHILAltitude(const RasterInfo *info)
{
  switch (info->system_id) {
  case UNKNOWN_SYSTEM:
    return 0;
  case SINGAPORE:
    return SINGAPORE_ALT;
  default:
    return CHIL_ALT;
  }
}

char *
CHILLocation(const RasterInfo *info)
{
  switch (info->system_id) {
  case UNKNOWN_SYSTEM:
    return "Unknown";
  case SINGAPORE:
    return "Singapore";
  default:
    return "Chilbolton";
  }
}
